@@ -45,6 +45,7 @@ typedef struct {
 	double x, y;
 	
 	/* Scale factor (stretches icon). */
+	/* ALEX TODO: only allow once scale */
 	double scale_x, scale_y;
 
 	/* Whether this item is selected. */
@@ -62,7 +63,7 @@ typedef struct {
 
 	double start_x, start_y;
 
-	GnomeCanvasItem *selection_rectangle;
+	EelCanvasItem *selection_rectangle;
 
 	guint timer_id;
 
@@ -156,7 +157,7 @@ struct NautilusIconContainerDetails {
 
 	/* Renaming Details */
 	gboolean renaming;
-	GnomeIconTextItem *rename_widget;	/* Editable text item */
+	GtkWidget *rename_widget;	/* Editable text item */
 	char *original_text;			/* Copy of editable text for later compare */
 
 	/* typeahead selection state */
