@@ -103,7 +103,7 @@ nautilus_link_desktop_file_local_create (const char        *directory_uri,
 					 const GdkPoint    *point,
 					 NautilusLinkType   type)
 {
-	char *uri, *contents, *escaped_name;
+	char *uri, *contents;
 	GnomeDesktopItem *desktop_item;
 	GList dummy_list;
 	NautilusFileChangesQueuePosition item;
@@ -112,9 +112,7 @@ nautilus_link_desktop_file_local_create (const char        *directory_uri,
 	g_return_val_if_fail (name != NULL, FALSE);
 	g_return_val_if_fail (target_uri != NULL, FALSE);
 
-	escaped_name = gnome_vfs_escape_string (name);
-	uri = g_strdup_printf ("%s/%s", directory_uri, escaped_name);
-	g_free (escaped_name);
+	uri = g_strdup_printf ("%s/%s", directory_uri, name);
 
 	contents = g_strdup_printf ("[Desktop Entry]\n"
 				    "Encoding=UTF-8\n"
@@ -338,7 +336,7 @@ nautilus_link_desktop_file_get_link_name_from_desktop (GnomeDesktopItem *desktop
 static char *
 nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop_file)
 {
-	GnomeIconLoader *icon_loader;
+	GnomeIconTheme *icon_theme;
 	char *icon_uri;
 	char *absolute;
 
@@ -348,8 +346,8 @@ nautilus_link_desktop_file_get_link_icon_from_desktop (GnomeDesktopItem *desktop
 	}
 
 	/* Fall back to a standard icon. */
-	icon_loader = nautilus_icon_factory_get_icon_loader ();
-	absolute = gnome_desktop_item_get_icon (desktop_file, icon_loader);
+	icon_theme = nautilus_icon_factory_get_icon_theme ();
+	absolute = gnome_desktop_item_get_icon (desktop_file, icon_theme);
 	
 	if (absolute == NULL) {
 		return NULL;
