@@ -32,7 +32,6 @@
 #include "nautilus-property-browser.h"
 
 #include "nautilus-signaller.h"
-#include <eel/eel-background.h>
 #include <eel/eel-gdk-extensions.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-glib-extensions.h>
@@ -49,6 +48,7 @@
 #include <libxml/parser.h>
 #include <gtk/gtkalignment.h>
 #include <gtk/gtkcolorseldialog.h>
+#include <gtk/gtkmessagedialog.h>
 #include <gtk/gtkdnd.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtkeventbox.h>
@@ -63,10 +63,12 @@
 #include <gtk/gtkstock.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtktogglebutton.h>
+#include <gtk/gtkradiobutton.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkviewport.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-util.h>
+#include <libgnome/gnome-help.h>
 #include <libgnomeui/gnome-color-picker.h>
 #include <libgnomeui/gnome-icon-entry.h>
 #include <libgnomeui/gnome-stock-icons.h>
@@ -81,6 +83,7 @@
 #include <libnautilus-private/nautilus-metadata.h>
 #include <libnautilus-private/nautilus-theme.h>
 #include <math.h>
+#include <atk/atkrelationset.h>
 
 /* property types */
 
@@ -100,7 +103,6 @@ struct NautilusPropertyBrowserDetails {
 	
 	GtkWidget *category_container;
 	GtkWidget *category_box;
-	GtkWidget *selected_button;
 	
 	GtkWidget *title_box;
 	GtkWidget *title_label;
@@ -143,7 +145,6 @@ struct NautilusPropertyBrowserDetails {
 	gboolean remove_mode;
 	gboolean keep_around;
 	gboolean has_local;
-	gboolean toggle_button_flag;
 };
 
 static void     nautilus_property_browser_class_init      (GtkObjectClass                *object_klass);
@@ -161,6 +162,8 @@ static void     add_new_button_callback                         (GtkWidget
 static void     cancel_remove_mode                              (NautilusPropertyBrowser       *property_browser);
 static void     done_button_callback                            (GtkWidget                     *widget,
 								 GtkWidget                     *property_browser);
+static void     help_button_callback                            (GtkWidget                     *widget,
+								 GtkWidget                     *property_browser);
 static void     remove_button_callback                          (GtkWidget                     *widget,
 								 NautilusPropertyBrowser       *property_browser);
 static gboolean nautilus_property_browser_delete_event_callback (GtkWidget                     *widget,
@@ -186,10 +189,6 @@ static void     element_clicked_callback                        (GtkWidget
 								 gpointer                       callback_data);
 
 
-#define BROWSER_BACKGROUND_COLOR "#FFFFFF"
-
-#define THEME_SELECT_COLOR "#FF9999"
-
 #define BROWSER_CATEGORIES_FILE_NAME "browser.xml"
 
 #define PROPERTY_BROWSER_WIDTH 540
@@ -241,7 +240,7 @@ static void
 nautilus_property_browser_init (GtkObject *object)
 {
  	NautilusPropertyBrowser *property_browser;
- 	GtkWidget *widget, *temp_box, *temp_hbox, *temp_frame;
+ 	GtkWidget *widget, *temp_box, *temp_hbox, *temp_frame, *vbox;
 	GtkWidget *temp_button, *align;
 	GtkWidget *viewport;
 	char *temp_str;
@@ -260,20 +259,28 @@ nautilus_property_browser_init (GtkObject *object)
 	g_free (temp_str);
 	
 	/* set the initial size of the property browser */
-	gtk_widget_set_size_request (widget, PROPERTY_BROWSER_WIDTH, PROPERTY_BROWSER_HEIGHT);
+	gtk_window_set_default_size (GTK_WINDOW (property_browser),
+				     PROPERTY_BROWSER_WIDTH,
+				     PROPERTY_BROWSER_HEIGHT);
 	gtk_container_set_border_width (GTK_CONTAINER (widget), 0);				
 
 	/* set the title and standard close accelerator */
 	gtk_window_set_title (GTK_WINDOW (widget), _("Backgrounds and Emblems"));
 	gtk_window_set_wmclass (GTK_WINDOW (widget), "property_browser", "Nautilus");
 	eel_gtk_window_set_up_close_accelerator (GTK_WINDOW (widget));
-		
+
+	/* create the main vbox. */
+  	vbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (vbox);
+	gtk_container_add (GTK_CONTAINER (property_browser), vbox);
+
 	/* create the container box */  
   	property_browser->details->container = GTK_HBOX (gtk_hbox_new (FALSE, 0));
 	gtk_container_set_border_width (GTK_CONTAINER (property_browser->details->container), 0);
 	gtk_widget_show (GTK_WIDGET (property_browser->details->container));
-	gtk_container_add (GTK_CONTAINER (property_browser),
-			   GTK_WIDGET (property_browser->details->container));	
+	gtk_box_pack_start (GTK_BOX (vbox),
+			    GTK_WIDGET (property_browser->details->container),
+			    TRUE, TRUE, 0);	
 
 	/* make the category container */
 	property_browser->details->category_container = gtk_scrolled_window_new (NULL, NULL);
@@ -335,7 +342,6 @@ nautilus_property_browser_init (GtkObject *object)
  	/* add the help label */
 	property_browser->details->help_label = gtk_label_new  ("");
 	gtk_widget_show(property_browser->details->help_label);
-	eel_gtk_label_set_scale (GTK_LABEL (property_browser->details->help_label), PANGO_SCALE_SMALL);
 	gtk_box_pack_end (GTK_BOX (temp_hbox), property_browser->details->help_label, FALSE, FALSE, 8);
  	 	
   	/* add the bottom box to hold the command buttons */
@@ -352,16 +358,23 @@ nautilus_property_browser_init (GtkObject *object)
 	gtk_widget_show (property_browser->details->bottom_box);
 	gtk_container_set_border_width (GTK_CONTAINER (property_browser->details->bottom_box), GNOME_PAD_SMALL);
 	
-	gtk_box_pack_end (GTK_BOX(property_browser->details->content_container), temp_box, FALSE, FALSE, 0);
+	gtk_box_pack_end (GTK_BOX (vbox), temp_box, FALSE, FALSE, 0);
   	gtk_container_add (GTK_CONTAINER (temp_frame), property_browser->details->bottom_box);
-  	
+  
+  	/* create the "help" button */
+	temp_button = gtk_button_new_from_stock (GTK_STOCK_HELP);
+
+	gtk_widget_show (temp_button);
+	gtk_box_pack_start (GTK_BOX (property_browser->details->bottom_box), temp_button, FALSE, FALSE, GNOME_PAD_SMALL);
+	g_signal_connect_object (temp_button, "clicked", G_CALLBACK (help_button_callback), property_browser, 0);
+	
   	/* create the "done" button */
  	temp_button = gtk_button_new_from_stock (GTK_STOCK_CLOSE);
 	GTK_WIDGET_SET_FLAGS (temp_button, GTK_CAN_DEFAULT);
 	eel_gtk_button_set_padding (GTK_BUTTON (temp_button), GNOME_PAD_SMALL);
 
-	gtk_widget_show(temp_button);
-	gtk_box_pack_end (GTK_BOX(property_browser->details->bottom_box), temp_button, FALSE, FALSE, GNOME_PAD_SMALL);
+	gtk_widget_show (temp_button);
+	gtk_box_pack_end (GTK_BOX (property_browser->details->bottom_box), temp_button, FALSE, FALSE, GNOME_PAD_SMALL);
 	gtk_widget_grab_default (temp_button);
 	gtk_widget_grab_focus (temp_button);
  	g_signal_connect_object (temp_button, "clicked", G_CALLBACK (done_button_callback), property_browser, 0);
@@ -481,7 +494,6 @@ nautilus_property_browser_destroy (GtkObject *object)
 		main_browser = NULL;
 		
 	EEL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
-
 }
 
 /* create a new instance */
@@ -761,30 +773,15 @@ make_color_drag_image (NautilusPropertyBrowser *property_browser, const char *co
 static void
 category_clicked_callback (GtkWidget *widget, char *category_name)
 {
-	gboolean save_flag;
 	NautilusPropertyBrowser *property_browser;
 	
 	property_browser = NAUTILUS_PROPERTY_BROWSER (g_object_get_data (G_OBJECT (widget), "user_data"));
-	
-	/* special case the user clicking on the already selected button, since we don't want that to toggle */
-	if (widget == GTK_WIDGET(property_browser->details->selected_button)) {
-		if (!property_browser->details->toggle_button_flag)
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (property_browser->details->selected_button), TRUE);		
-		return;
-	}	
 
 	/* exit remove mode when the user switches categories, since there might be nothing to remove
 	   in the new category */
 	property_browser->details->remove_mode = FALSE;
 		
-	save_flag = property_browser->details->toggle_button_flag;
-	property_browser->details->toggle_button_flag = TRUE;	
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (property_browser->details->selected_button), FALSE);
-	property_browser->details->toggle_button_flag = save_flag;	
-	
 	nautilus_property_browser_set_category (property_browser, category_name);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), TRUE);
-	property_browser->details->selected_button = widget;
 }
 
 static xmlDocPtr
@@ -809,7 +806,7 @@ write_browser_xml (NautilusPropertyBrowser *property_browser,
 	char *user_directory, *path;
 
 	user_directory = nautilus_get_user_directory ();	
-	path = nautilus_make_path (user_directory, property_browser->details->path);
+	path = g_build_filename (user_directory, property_browser->details->path, NULL);
 	g_free (user_directory);
 	xmlSaveFile (path, document);
 	g_free (path);
@@ -1106,10 +1103,10 @@ add_pattern_to_browser (const char *path_name, gpointer *data)
 	user_directory = nautilus_get_user_directory ();
 		
 	/* copy the image file to the patterns directory */
-	directory_path = nautilus_make_path (user_directory, "patterns");
+	directory_path = g_build_filename (user_directory, "patterns", NULL);
 	g_free (user_directory);
 	source_file_name = strrchr (path_name, '/');
-	destination_name = nautilus_make_path (directory_path, source_file_name + 1);
+	destination_name = g_build_filename (directory_path, source_file_name + 1, NULL);
 
 	/* make the directory if it doesn't exist */
 	if (!g_file_test(directory_path, G_FILE_TEST_EXISTS)) {
@@ -1373,8 +1370,9 @@ emblem_dialog_clicked (GtkWidget *dialog, int which_button, NautilusPropertyBrow
 			user_directory = nautilus_get_user_directory ();
 
 			/* get the path for emblems in the user's home directory */
-			directory_path = nautilus_make_path (user_directory,
-							     "emblems");
+			directory_path = g_build_filename (user_directory,
+							   "emblems",
+							   NULL);
 			g_free (user_directory);
 
 			/* make the directory if it doesn't exist */
@@ -1481,6 +1479,34 @@ done_button_callback (GtkWidget *widget, GtkWidget *property_browser)
 	gtk_widget_hide (property_browser);
 }
 
+/* handle the "help" button */
+static void
+help_button_callback (GtkWidget *widget, GtkWidget *property_browser)
+{
+	GError *error = NULL;
+	GtkWidget *dialog;
+
+	gnome_help_display_desktop (NULL,
+				    "user-guide",
+				    "wgosnautilus.xml", "gosnautilus-50",
+				    &error);
+	if (error) {
+		dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
+						 GTK_DIALOG_DESTROY_WITH_PARENT,
+						 GTK_MESSAGE_ERROR,
+						 GTK_BUTTONS_CLOSE,
+						 _("There was an error displaying help: \n%s"),
+						 error->message);
+
+		g_signal_connect (G_OBJECT (dialog),
+				  "response", G_CALLBACK (gtk_widget_destroy),
+				  NULL);
+		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+		gtk_widget_show (dialog);
+		g_error_free (error);
+	}
+}
+
 /* handle the "remove" button */
 static void
 remove_button_callback(GtkWidget *widget, NautilusPropertyBrowser *property_browser)
@@ -1867,7 +1893,9 @@ property_browser_category_button_new (const char *display_name,
 	file_name = nautilus_pixmap_file (image); 
 	g_return_val_if_fail (file_name != NULL, NULL);
 
-	button = eel_labeled_image_toggle_button_new_from_file_name (display_name, file_name);
+	button = eel_labeled_image_radio_button_new_from_file_name (display_name, file_name);
+
+	gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
 
 	/* We also want all of the buttons to be the same height */
 	eel_labeled_image_set_fixed_image_height (EEL_LABELED_IMAGE (GTK_BIN (button)->child), STANDARD_BUTTON_IMAGE_HEIGHT);
@@ -1882,24 +1910,30 @@ static void
 make_category_link (NautilusPropertyBrowser *property_browser,
 		    const char *name,
 		    const char *display_name,
-		    const char *image)
+		    const char *image,
+		    GtkRadioButton **group)
 {
 	GtkWidget *button;
 
-	g_return_if_fail (NAUTILUS_IS_PROPERTY_BROWSER (property_browser));
 	g_return_if_fail (name != NULL);
-	g_return_if_fail (display_name != NULL);
 	g_return_if_fail (image != NULL);
+	g_return_if_fail (display_name != NULL);
+	g_return_if_fail (NAUTILUS_IS_PROPERTY_BROWSER (property_browser));
 
 	button = property_browser_category_button_new (display_name, image);
 	gtk_widget_show (button);
+
+	if (*group) {
+		gtk_radio_button_set_group (GTK_RADIO_BUTTON (button),
+					    gtk_radio_button_get_group (*group));
+	} else {
+		*group = GTK_RADIO_BUTTON (button);
+	}
 	
 	/* if the button represents the current category, highlight it */	
 	if (property_browser->details->category &&
-	    strcmp (property_browser->details->category, name) == 0) {
+	    strcmp (property_browser->details->category, name) == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
-		property_browser->details->selected_button = button;		
-	}
 
 	/* Place it in the category box */
 	gtk_box_pack_start (GTK_BOX (property_browser->details->category_box),
@@ -1921,8 +1955,8 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 {
 	xmlNodePtr cur_node;
  	xmlDocPtr document;
- 	EelBackground *background;
 	GtkWidget *viewport;
+	GtkRadioButton *group;
 	gboolean got_categories;
 	char *name, *image, *type, *description, *display_name, *path, *mode;
 	const char *text;
@@ -1944,11 +1978,10 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
  	viewport = gtk_viewport_new (NULL, NULL);
 	gtk_widget_show(viewport);
 	gtk_viewport_set_shadow_type(GTK_VIEWPORT(viewport), GTK_SHADOW_IN);
-	background = eel_get_widget_background (viewport);
-	eel_background_set_color (background, BROWSER_BACKGROUND_COLOR);	
 	gtk_container_add (GTK_CONTAINER (property_browser->details->content_container), property_browser->details->content_frame);
 	gtk_widget_show (property_browser->details->content_frame);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (property_browser->details->content_frame), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (property_browser->details->content_frame),
+					GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
 	/* allocate a table to hold the content widgets */
   	property_browser->details->content_table = eel_image_table_new (TRUE);
@@ -1969,7 +2002,8 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 	if (!got_categories) {
 		property_browser->details->category_position = 0;
 	}
-	
+
+	group = NULL;
 	for (cur_node = eel_xml_get_children (xmlDocGetRootElement (document));
 	     cur_node != NULL;
 	     cur_node = cur_node->next) {
@@ -2008,7 +2042,8 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 				make_category_link (property_browser,
 						    name,
 						    display_name,
-						    image);
+						    image,
+						    &group);
 				
 				xmlFree (display_name);
 				xmlFree (image);
@@ -2099,7 +2134,8 @@ nautilus_property_browser_update_contents (NautilusPropertyBrowser *property_bro
 		}
 		
 		if (label_text) {
-			gtk_label_set_text_with_mnemonic (GTK_LABEL (property_browser->details->title_label), label_text);
+			gtk_label_set_text_with_mnemonic
+				(GTK_LABEL (property_browser->details->title_label), label_text);
 		}
 		g_free(label_text);
 
