@@ -32,14 +32,14 @@
 
 #include "nautilus-application.h"
 #include "nautilus-bookmarks-window.h"
+#include "nautilus-information-panel.h"
 #include "nautilus-main.h"
-#include "nautilus-sidebar.h"
 #include "nautilus-signaller.h"
 #include "nautilus-switchable-navigation-bar.h"
 #include "nautilus-window-manage-views.h"
 #include "nautilus-zoom-control.h"
-#include <X11/Xatom.h>
 #include <bonobo/bonobo-exception.h>
+#include <bonobo/bonobo-property-bag-client.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <eel/eel-debug.h>
 #include <eel/eel-gdk-extensions.h>
@@ -90,6 +90,10 @@
 #define STATUS_BAR_CLEAR_TIMEOUT 10000	/* milliseconds */
 #define MAX_HISTORY_ITEMS 50
 
+/* FIXME bugzilla.gnome.org 41245: hardwired sizes */
+#define SIDE_PANE_MINIMUM_WIDTH 1
+#define SIDE_PANE_MINIMUM_HEIGHT 400
+
 /* dock items */
 #define LOCATION_BAR_PATH	"/Location Bar"
 #define TOOLBAR_PATH            "/Toolbar"
@@ -112,6 +116,7 @@ enum {
 };
 
 static GList *history_list;
+static int side_pane_width_auto_value = SIDE_PANE_MINIMUM_WIDTH;
 
 static void update_sidebar_panels_from_preferences (NautilusWindow *window);
 static void sidebar_panels_changed_callback        (gpointer        user_data);
@@ -180,6 +185,9 @@ nautilus_window_instance_init (NautilusWindow *window)
 	g_object_ref (G_OBJECT (window->details->tooltips));
 	gtk_object_sink (GTK_OBJECT (window->details->tooltips));
 
+	/* Set last geometry to NULL */
+	window->last_geometry = NULL;
+
 	/* Set initial window title */
 	gtk_window_set_title (GTK_WINDOW (window), _("Nautilus"));
 
@@ -527,29 +535,146 @@ set_dummy_initial_view_as_menu (NautilusWindow *window)
                                   new_menu);
 }
 
+static void
+side_pane_close_requested_callback (GtkWidget *widget,
+				    gpointer user_data)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (user_data);
+
+	nautilus_window_hide_sidebar (window);
+}
+
+static void
+side_pane_size_allocate_callback (GtkWidget *widget,
+				  GtkAllocation *allocation,
+				  gpointer user_data)
+{
+	NautilusWindow *window;
+	
+	window = NAUTILUS_WINDOW (user_data);
+	
+	if (allocation->width != window->details->side_pane_width) {
+		window->details->side_pane_width = allocation->width;
+		eel_preferences_set_integer
+			(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH, 
+			 allocation->width);
+	}
+}
+
+static void
+setup_side_pane_width (NautilusWindow *window)
+{
+	static gboolean setup_auto_value= TRUE;
+
+	g_return_if_fail (window->sidebar != NULL);
+	
+	if (setup_auto_value) {
+		setup_auto_value = FALSE;
+		eel_preferences_add_auto_integer 
+			(NAUTILUS_PREFERENCES_SIDEBAR_WIDTH,
+			 &side_pane_width_auto_value);
+	}
+
+	window->details->side_pane_width = side_pane_width_auto_value;
+
+	/* FIXME bugzilla.gnome.org 41245: Saved in pixels instead of in %? */
+        /* FIXME bugzilla.gnome.org 41245: No reality check on the value? */
+	
+	gtk_paned_set_position (GTK_PANED (window->content_hbox), 
+				side_pane_width_auto_value);
+}
+
+static void
+side_panel_set_open (GtkWidget *view,
+		     gboolean open)
+{
+	CORBA_Environment ev;
+	Bonobo_PropertyBag property_bag;
+	Bonobo_Control control;
+	
+	if (!view || !NAUTILUS_IS_VIEW_FRAME (view)) {
+		return;
+	}
+
+	control = nautilus_view_frame_get_control (NAUTILUS_VIEW_FRAME (view));
+	
+	if (control != CORBA_OBJECT_NIL) {
+		CORBA_exception_init (&ev);
+		property_bag = Bonobo_Control_getProperties (control, &ev);
+		if (!BONOBO_EX (&ev) && property_bag != CORBA_OBJECT_NIL) {
+			/* For some reason this was implemented as 'close'
+			 * before, but open seems more natural */
+			bonobo_property_bag_client_set_value_gboolean
+				(property_bag, "close", !open, &ev);
+			bonobo_object_release_unref (property_bag, NULL);
+		}
+	}
+}
+
+static void
+side_pane_switch_page_callback (NautilusSidePane *side_pane,
+				GtkWidget *panel,
+				NautilusWindow *window)
+{
+	if (window->details->current_side_panel) {
+		side_panel_set_open (window->details->current_side_panel, 
+				     FALSE);
+		eel_remove_weak_pointer (&window->details->current_side_panel);
+	}
+
+	side_panel_set_open (panel, TRUE);	
+	window->details->current_side_panel = panel;
+	eel_add_weak_pointer (&window->details->current_side_panel);
+}
+
 static void
 nautilus_window_set_up_sidebar (NautilusWindow *window)
 {
-	window->sidebar = nautilus_sidebar_new ();
+	window->sidebar = nautilus_side_pane_new ();
+
+	g_signal_connect (window->sidebar,
+			  "close_requested",
+			  G_CALLBACK (side_pane_close_requested_callback),
+			  window);
+
+	g_signal_connect (window->sidebar,
+			  "switch_page",
+			  G_CALLBACK (side_pane_switch_page_callback),
+			  window);
 
-	if (window->details->location != NULL &&
-	    window->details->title != NULL) {
-		nautilus_sidebar_set_uri (window->sidebar,
-					  window->details->location,
-					  window->details->title);
-	}
-	
-	g_signal_connect_object (window->sidebar, "location_changed",
-				 G_CALLBACK (go_to_callback), window, 0);
 	gtk_paned_pack1 (GTK_PANED (window->content_hbox),
 			 GTK_WIDGET (window->sidebar),
 			 FALSE, TRUE);
+
+	setup_side_pane_width (window);
+	g_signal_connect (window->sidebar, 
+			  "size_allocate",
+			  G_CALLBACK (side_pane_size_allocate_callback),
+			  window);
 	
-	nautilus_sidebar_setup_width (window->sidebar);
+	window->information_panel = nautilus_information_panel_new ();
+	
+	if (window->details->location != NULL &&
+	    window->details->title != NULL) {
+		nautilus_information_panel_set_uri (window->information_panel,
+						    window->details->location,
+						    window->details->title);
+	}
+
+	g_signal_connect_object (window->information_panel, "location_changed",
+				 G_CALLBACK (go_to_callback), window, 0);
+
+	gtk_widget_show (GTK_WIDGET (window->information_panel));
+
+	nautilus_side_pane_add_panel (NAUTILUS_SIDE_PANE (window->sidebar), 
+				      GTK_WIDGET (window->information_panel),
+				      _("Information"));
 
 	/* Set up the sidebar panels. */
 	update_sidebar_panels_from_preferences (window);
-
+	
 	gtk_widget_show (GTK_WIDGET (window->sidebar));
 }
 
@@ -559,6 +684,7 @@ nautilus_window_tear_down_sidebar (NautilusWindow *window)
 	nautilus_window_set_sidebar_panels (window, NULL);
 	gtk_widget_destroy (GTK_WIDGET (window->sidebar));
 	window->sidebar = NULL;
+	window->information_panel = NULL;
 }
 
 static void
@@ -786,10 +912,16 @@ nautilus_window_destroy (GtkObject *object)
 
 	nautilus_window_manage_views_destroy (window);
 
+	window->sidebar = NULL;
 	eel_g_object_list_free (window->sidebar_panels);
 	window->sidebar_panels = NULL;
 
-	if (window->content_view != NULL) {
+	window->view_as_option_menu = NULL;
+	window->navigation_bar = NULL;
+	window->content_hbox = NULL;
+	window->zoom_control = NULL;
+
+	if (window->content_view) {
 		gtk_object_destroy (GTK_OBJECT (window->content_view));
 		window->content_view = NULL;
 	}
@@ -864,35 +996,27 @@ nautilus_window_finalize (GObject *object)
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
+void
 nautilus_window_save_geometry (NautilusWindow *window)
 {
 	char *geometry_string;
 
-        g_assert (NAUTILUS_IS_WINDOW (window));
-	g_assert (GTK_WIDGET_VISIBLE (window));
+	g_assert (NAUTILUS_IS_WINDOW (window));
 
-        geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
+	geometry_string = eel_gtk_window_get_geometry_string (GTK_WINDOW (window));
 
 	nautilus_file_set_metadata (window->details->viewed_file,
 				    NAUTILUS_METADATA_KEY_WINDOW_GEOMETRY,
 				    NULL,
 				    geometry_string);
+				    
 	g_free (geometry_string);
 }
 
 void
 nautilus_window_close (NautilusWindow *window)
 {
-        g_return_if_fail (NAUTILUS_IS_WINDOW (window));
-
-	/* Save the window position in the directory's metadata only if
-	 * we're in every-location-in-its-own-window mode. Otherwise it
-	 * would be too apparently random when the stored positions change.
-	 */
-	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_WINDOW_ALWAYS_NEW)) {
-	        nautilus_window_save_geometry (window);
-	}
+	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 
 	gtk_widget_destroy (GTK_WIDGET (window));
 }
@@ -1401,12 +1525,20 @@ void
 nautilus_window_add_sidebar_panel (NautilusWindow *window,
 				   NautilusViewFrame *sidebar_panel)
 {
+	char *label;
+
 	g_return_if_fail (NAUTILUS_IS_WINDOW (window));
 	g_return_if_fail (NAUTILUS_IS_VIEW_FRAME (sidebar_panel));
-	g_return_if_fail (NAUTILUS_IS_SIDEBAR (window->sidebar));
+	g_return_if_fail (NAUTILUS_IS_SIDE_PANE (window->sidebar));
 	g_return_if_fail (g_list_find (window->sidebar_panels, sidebar_panel) == NULL);
 	
-	nautilus_sidebar_add_panel (window->sidebar, sidebar_panel);
+	label = nautilus_view_frame_get_label (sidebar_panel);
+	
+	nautilus_side_pane_add_panel (window->sidebar, 
+				      GTK_WIDGET (sidebar_panel), 
+				      label);
+
+	g_free (label);
 
 	g_object_ref (sidebar_panel);
 	window->sidebar_panels = g_list_prepend (window->sidebar_panels, sidebar_panel);
@@ -1422,7 +1554,8 @@ nautilus_window_remove_sidebar_panel (NautilusWindow *window, NautilusViewFrame
 		return;
 	}
 	
-	nautilus_sidebar_remove_panel (window->sidebar, sidebar_panel);
+	nautilus_side_pane_remove_panel (window->sidebar, 
+					 GTK_WIDGET (sidebar_panel));
 	window->sidebar_panels = g_list_remove (window->sidebar_panels, sidebar_panel);
 	g_object_unref (sidebar_panel);
 }
@@ -1499,7 +1632,11 @@ nautilus_window_go_home (NautilusWindow *window)
 
 	nautilus_window_set_search_mode (window, FALSE);
 
+#ifdef WEB_NAVIGATION_ENABLED
 	home_uri = eel_preferences_get (NAUTILUS_PREFERENCES_HOME_URI);
+#else
+	home_uri = gnome_vfs_get_uri_from_local_path (g_get_home_dir ());
+#endif
 	
 	g_assert (home_uri != NULL);
 	nautilus_window_go_to (window, home_uri);
